////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//   Wireless Communication Library 7                                         //
//                                                                            //
//   Copyright (C) 2006-2024 Mike Petrichenko                                 //
//                           Soft Service Company                             //
//                           All Rights Reserved                              //
//                                                                            //
//   http://www.btframework.com                                               //
//                                                                            //
//   support@btframework.com                                                  //
//   shop@btframework.com                                                     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <list>

#include "..\Common\wclHelpers.h"
#include "wclTimelineErrors.h"

/// <summary> This macro helps to build correct documentation. </summary>
#define _WCL_TIMELINE_UAAPI_INCLUDE_FILE_ "wclUaApi.h"
#include _WCL_TIMELINE_UAAPI_INCLUDE_FILE_

using namespace wclCommon;

namespace wclTimeline
{
	// Forward declaration.
	class CwclUserActivity;

	///<summary> The TimelineChannel component provides access to the Windows
	///   Timeline graph. </summary>
	/// <remarks> The Windows Timeline is supported on Windows 10 16299 and
	///   above. </remarks>
	class CwclTimelineChannel
	{
		DISABLE_COPY(CwclTimelineChannel);

	private:
		// User Activities channel.
		wclTimeline::UaApi::IUserActivityChannel*	FChannel;
		
		/* WinRT initialization and support. */
		
		HANDLE	FWinRtInitEvent;
		int		FWinRtInitResult;
		HANDLE	FWinRtTermEvent;
		HANDLE	FWinRtThread;
		
		/* WinRT support thread. */
		
		static UINT __stdcall _WinRtThread(LPVOID lpParam);
		void WinRtThread();
		
		/* Helper functions. */
		
		static int CheckVersion();
		static int DeleteKey(const tstring& Key);
		
	public:
		/// <summary> Creates new component. </summary>
		CwclTimelineChannel();
		/// <summary> Frees the component. </summary>
		virtual ~CwclTimelineChannel();
		
		/// <summary> Closes the Timeline Channel. </summary>
		/// <returns> If the function succeed the return value is
		///   <see cref="WCL_E_SUCCESS" />. Otherwise the method returns one of
		///   the WCL error codes. </returns>
		/// <remarks> An application is responsible to release all created or
		///   enumerated Activities before closing the Timeline Channel. </remarks>
		int Close();
		/// <summary> Opens the default Timeline Channel associated with
		///   the user's Managed Service Account (MSA). </summary>
		/// <returns> If the function succeed the return value is
		///   <see cref="WCL_E_SUCCESS" />. Otherwise the method returns one of
		///   the WCL error codes. </returns>
		/// <remarks> Only one active Timeline Channel is supported. </remarks>
		int Open();
		
		/// <summary> Creates (or gets) a User Activity with the specified
		///   ID. </summary>
		/// <param name="Id"> The identifier for the User Activity. </param>
		/// <param name="Activity"> <para> If the method completed with success on
		///   output contains the created User Activity object. </para>
		///   <para> An application is responsible to release the User Activity
		///   object returned by this method. </para> </param>
		/// <returns> If the function succeed the return value is
		///   <see cref="WCL_E_SUCCESS" />. Otherwise the method returns one of
		///   the WCL error codes. </returns>
		/// <remarks> <para> Your application should name activities in such a way
		///   that same ID is generated each time the user is in a particular
		///   location in the app. For example, if your application is page-based,
		///   use an identifier for the page, if its document based, use the name
		///   of the doc (or a hash of the name). </para>
		///   <para> If there is an existing activity in the feed with the same ID,
		///   that activity will be return from the channel (with the UserActivity
		///   object State property set to <c>asPublished</c>). If there is no
		///   activity with that name, and new activity with State set to
		///   <c>asNew</c>. </para>
		///   <para> Activities are scoped to your app, there is no need to worry
		///   about your activity ID colliding with IDs from other
		///   applications. </para> </remarks>
		/// <seealso cref="CwclUserActivity" />
		int CreateActivity(const tstring& Id, CwclUserActivity*& Activity);
		
		/// <summary> Registers an Application Activation Protocol. </summary>
		/// <param name="Protocol"> The string represents the unique identifier for
		///   the Application Activation Protocol. </param>
		/// <param name="DisplayName"> The user friendly display name that describes
		///   the Application Activation protocol. </param>
		/// <returns> If the function succeed the return value is
		///   <see cref="WCL_E_SUCCESS" />. Otherwise the method returns one of
		///   the WCL error codes. </returns>
		/// <remarks> <para> The Application Activation Protocol is a part of the
		///   Activation Deep Link, a URI that will be used to start your
		///   application to resume the application with specific context. It is
		///   used to activate your application when user selects activity from the
		///   Windows Timeline Graph. </para>
		///   <para> The Application Activation Protocol must unique identifies your
		///   application and must not overwrite existing system protocol. An
		///   application can register more than one protocol. </para> </remarks>
		static int RegisterProtocol(const tstring& Protocol, const tstring& DisplayName);
		/// <summary> Unregisters the Application Activation Protocol. </summary>
		/// <param name="Protocol"> The string represents the unique identifier for
		///   the Application Activation Protocol. </param>
		/// <returns> If the function succeed the return value is
		///   <see cref="WCL_E_SUCCESS" />. Otherwise the method returns one of
		///   the WCL error codes. </returns>
		/// <remarks> The <c>Protocol</c> must be the same that was used in the
		///   <c>RegisterProtocol</c> call. </remarks>
		static int UnregisterProtocol(const tstring& Protocol);
		
		/// <summary> Gets the current state of the Windows Timeline
		///   Channel. </summary>
		/// <returns> <c>True</c> if the Timeline Channel is opened. <c>False</c>
		///   otherwise. </returns>
		bool GetActive() const;
		/// <summary> Gets the current state of the Windows Timeline
		///   Channel. </summary>
		/// <value> <c>True</c> if the Timeline Channel is opened. <c>False</c>
		///   otherwise. </value>
		__declspec(property(get = GetActive)) bool Active;
	};

	/// <summary> Defines the states of a User Activity. </summary>
	typedef enum
	{
		/// <summary> The User Activity has not yet been published. </summary>
		asNew,
		/// <summary> The User Activity has been published on this (or another)
		///   device. </summary>
		asPublished
	} wclUserActivityState;

	/// <summary> <para> The class represents a User Activity. </para>
	///   <para> A User Activity is created by an application during its execution
	///   to notify the system of a user work stream that can be continued at
	///   another time. It provides information about a task the user is engaged
	///   in. </para> </summary>
	/// <remarks> <para> A User Activity encapsulates a user's task that can be
	///   continued at a later time. Creating a User Activity causes that activity
	///   to appear in Windows Timeline and in Cortana's Pick up where I left off
	///   feature. Timeline is a rich task view that shows a chronological view of
	///   what youve been working on. It can also include what you were working
	///   on across devices. For example, a mail app could create a User Activity
	///   when the user starts creating a new email message. The user could pause
	///   working on the email and then work on it later on the same machine, or
	///   even another device. </para>
	///   <para> An application must never create this object directly. Instead of
	///   that an application must use <see cref="CwclTimelineChannel" />
	///   component to create or get an User Activity objects. </para>
	///   <para> The User Activity is supported on Windows 10 16299 and
	///   above. </para> </remarks>
	/// <seealso cref="CwclTimelineChannel" />
	class CwclUserActivity
	{
		DISABLE_COPY(CwclUserActivity);
		
	private:
		friend class CwclTimelineChannel;

		tstring					FDescription;
		tstring					FDisplayText;
		tstring					FId;
		tstring					FParams;
		tstring					FProtocol;
		wclUserActivityState	FState;
		tstring					FTitle;
		
		wclTimeline::UaApi::IUserActivity*					FActivity;
		wclTimeline::UaApi::IUserActivitySession*			FSession;
		wclTimeline::UaApi::IUserActivityVisualElements*	FVisualElements;
		wclTimeline::UaApi::IUserActivityVisualElements2*	FVisualElements2;
		
		// For internal use. Called by the TwclTimelineChannel when initializes
		// the User Activity object.
		int Init(wclTimeline::UaApi::IUserActivity* const Activity);
		
	public:
		/// <summary> Creates new User Activity object. </summary>
		/// <remarks> <para> An application must never create the User Activity
		///   object directly. Instead of that an application must use high level
		///   <see cref="CwclTimelineChannel" /> object to create it. </para>
		///   <para> An application is responsible to free the User Activity object
		///   when finished work with this. </para>
		///   <para> An application must free the USer Activity object before
		///   it closes the Timeline Channel component. </para> </remarks>
		/// <seealso cref="CwclTimelineChannel" />
		/// <exception cref="wclEInvalidArgument"></exception>
		CwclUserActivity();
		/// <summary> Frees the object. </summary>
		/// <remarks> An application is responsible to free the User Activity object
		///   when finished work with this. </remarks>
		virtual ~CwclUserActivity();
		
		/// <summary> Starts User Activity session. </summary>
		/// <returns> If the function succeed the return value is
		///   <see cref="WCL_E_SUCCESS" />. Otherwise the method returns one of
		///   the WCL error codes. </returns>
		int BeginSession();
		/// <summary> Ends User Activity session. </summary>
		/// <returns> If the function succeed the return value is
		///   <see cref="WCL_E_SUCCESS" />. Otherwise the method returns one of
		///   the WCL error codes. </returns>
		int EndSession();
		
		/// <summary> Gets the USer Activity Session state. </summary>
		/// <returns> The session state. <c>True</c> if the USer Activity session is
		///   active. <c>False</c> otherwise. </returns>
		bool GetActive() const;
		/// <summary> Gets the USer Activity Session state. </summary>
		/// <value> The session state. <c>True</c> if the USer Activity session is
		///   active. <c>False</c> otherwise. </value>
		__declspec(property(get = GetActive)) bool Active;
		
		/// <summary> Gets the description text that is used for the
		///   details tile for this User Activity. </summary>
		/// <returns> The User Activity description. </returns>
		tstring GetDescription() const;
		/// <summary> Sets the description text that is used for the
		///   details tile for this User Activity. </summary>
		/// <param name="value"> The User Activity description. </param>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		void SetDescription(const tstring& value);
		/// <summary> Gets and sets the description text that is used for the
		///   details tile for this User Activity. </summary>
		/// <value> The User Activity description. </value>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		__declspec(property(get = GetDescription, put = SetDescription)) tstring Description;
		
		/// <summary> Gets the display text that is used for the details
		///   tile text for this User Activity. </summary>
		/// <returns> The display text. </returns>
		tstring GetDisplayText() const;
		/// <summary> Sets the display text that is used for the details
		///   tile text for this User Activity. </summary>
		/// <param name="value"> The display text. </param>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		void SetDisplayText(const tstring& value);
		/// <summary> Gets and sets the display text that is used for the details
		///   tile text for this User Activity. </summary>
		/// <value> The display text. </value>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		__declspec(property(get = GetDisplayText, put = SetDisplayText)) tstring DisplayText;
		
		/// <summary> Gets the Activity ID that was assigned to this User Activity
		///   when it was created. </summary>
		/// <returns> The activity ID. </returns>
		tstring GetId() const;
		/// <summary> Gets the Activity ID that was assigned to this User Activity
		///   when it was created. </summary>
		/// <value> The activity ID. </value>
		__declspec(property(get = GetId)) tstring Id;
		
		/// <summary> Gets the application activation parameters. </summary>
		/// <returns> The application activation parameters associated with this
		///   Activity. This parameters will be passed to the application when it
		///   activated when user clicked on its Activity. </returns>
		tstring GetParams() const;
		/// <summary> Sets the application activation parameters. </summary>
		/// <param name="value"> The application activation parameters associated with this
		///   Activity. This parameters will be passed to the application when it
		///   activated when user clicked on its Activity. </param>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		void SetParams(const tstring& value);
		/// <summary> Gets and sets the application activation
		///   parameters. </summary>
		/// <value> The application activation parameters associated with this
		///   Activity. This parameters will be passed to the application when it
		///   activated when user clicked on its Activity. </value>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		__declspec(property(get = GetParams, put = SetParams)) tstring Params;
		
		/// <summary> Gets the Activity's activation protocol. </summary>
		/// <returns> The activation protocol associated with this User Activity
		///   object. The protocol must be one of the registered application's
		///   protocols. </returns>
		tstring GetProtocol() const;
		/// <summary> Sets the Activity's activation protocol. </summary>
		/// <param name="value"> The activation protocol associated with this User Activity
		///   object. The protocol must be one of the registered application's
		///   protocols. </param>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		void SetProtocol(const tstring& value);
		/// <summary> Gets and sets the Activity's activation protocol. </summary>
		/// <value> The activation protocol associated with this User Activity
		///   object. The protocol must be one of the registered application's
		///   protocols. </value>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		__declspec(property(get = GetProtocol, put = SetProtocol)) tstring Protocol;
		
		/// <summary> Gets the state of this User Activity. </summary>
		/// <returns> The current User Activity state. </returns>
		/// <seealso cref="wclUserActivityState" />
		wclUserActivityState GetState() const;
		/// <summary> Gets the state of this User Activity. </summary>
		/// <value> The current User Activity state. </value>
		/// <seealso cref="wclUserActivityState" />
		__declspec(property(get = GetState)) wclUserActivityState State;

		/// <summary> Gets the text which is shown in the top banner of the
		///   activity card. </summary>
		/// <returns> The text that will appear in the top banner of the activity
		///   card. </returns>
		tstring GetTitle() const;
		/// <summary> Set the text which is shown in the top banner of the
		///   activity card. </summary>
		/// <param name="value"> The text that will appear in the top banner of the activity
		///   card. </param>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		void SetTitle(const tstring& value);
		/// <summary> Gets and set the text which is shown in the top banner of the
		///   activity card. </summary>
		/// <value> The text that will appear in the top banner of the activity
		///   card. </value>
		/// <remarks> If an application changes this property when User Activity
		///   session is active the <see cref="wclEUserActivitySessionActive" />
		///   exception raises. </remarks>
		/// <exception cref="wclEUserActivitySessionActive"></exception>
		/// <exception cref="wclEUserActivityNotInitialized"></exception>
		__declspec(property(get = GetTitle, put = SetTitle)) tstring Title;
	};
}